function [A,b] = Assembler_4(Mesh,function_spec)

    % Copyright (c) 2015-2023 Martin Neuenhofen. 
    
    %% Description
    %
    % For enriched mesh with boundary information and full function 
    % specification this routine assembles stiffness matrix A and
    % right-hand-side b of finite element discretization.
    %
    % A = int_Omega a(u,v) dx
    % b = int_Omega f(v)   dx + int_dOmegaNeumann g(v) ds
    %
    % Copyright (c) 2015-2023 Martin Neuenhofen. See COPYRIGHT for further
    % information.
    

    %% Beschreibung
    %
    % Eingaben:
    %   function_spec.a         % Funktion der Bilinearform
    %                           % Layout: a(phi_test,Dphi_test,phi_ans,Dphi_ans,x,h)
    %   function_spec.f         % Funktion der Linearform, Innen-Integral
    %                           % Layout: f(phi_test,Dphi_test,x,h)
    %   function_spec.g         % Funktion der Linearform, Randintegral
    %                           % Layout: f(phi_test,Dphi_test,x,h_Rand)
    %   function_spec.deg_Ans   % Grad der Ansatzfunktionen.    Bisher aus {1,2}
    %   function_spec.deg_Tes   % Grad der Testfunktionen.      Bisher aus {1,2}
    % 
    %   Mesh.p              	% Tetraheder-Eckpunkte
    %   Mesh.t                  % Liste der Tetraheder
    %   Mesh.bound.all          % Fr jedes p(i,:), ob es auf dem Gebietsrand liegt .
    %   Mesh.bound.Diri         % Fr jedes p(i,:), ob es auf dem Dirichlet-Rand liegt.
    %   Mesh.bound.Neum         % Fr jedes p(i,:), ob es auf dem Neumann-Rand liegt.
    %
    %  optional:
    %   Mesh.p2                 % alle Freiheitsgrade (nur notwendig fr Pk-Elemente, k > 1)
    %   Mesh.t2                 % Dreiecksliste, die vollstndig auf p2 verweist.
    %   Mesh.bound.all2         % Fr jedes p2(i,:), ob es auf dem Gebietsrand liegt .
    %   Mesh.bound.Diri2        % Fr jedes p2(i,:), ob es auf dem Dirichlet-Rand liegt.
    %   Mesh.bound.Neum2        % Fr jedes p2(i,:), ob es auf dem Neumann-Rand liegt.
    %
    % Vorher:
    %   Das Variationsproblem ist durch function_spec festgelegt.
    %   Die Diskretisierung und das numerische Gebiet inkl. Rnder sind durch Mesh festgelegt.
    %   Die Grade der jeweiligen Ansatzrume sind durch function_spec.deg_ festgelegt.
    % 
    % Nachher:
    %   Das Programm hat das LGS der Galerkin-Diskretisierung des
    %   Variationsproblems nach function_spec auf dem durch Mesh
    %   festgelegten endlich-dimensionalen Ansatzraum assembliert.
    %
    % Ausgaben:
    %   A                       % a(phi_Tes_i,phi_Ans_j), i Zeile, j Spalte
    %                           % Alle homogenen Dirichlet-Rnder wurden bereits aus Zeilen und Spalten enfernt.
    %                           % Die Funktionen phi_Tes_i,phi_Ans_j sind nodale Basisfunktionen in der Reihenfolge
    %                           % der Knoten aus der Liste p2.
    %   b                       % f(phi_Tes_i), i Zeile
    %                           % Alle homogenen Dirichlet-Rnder wurden bereits aus Zeilen und Spalten enfernt.
    %                           % Die Funktionen phi_Tes_i sind nodale Basisfunktionen in der Reihenfolge
    %                           % der Knoten aus der Liste p2.
    %
    % %


    global func_a func_f func_g

    %% Eingaben entpacken:
    
    % das numerische Gitter: Knoten, Tetraheder, Randknoten
    t   = Mesh.t;
    p   = Mesh.p;
    
    if(isfield(Mesh,'t2'))
        t2                  = Mesh.t2;
        p2                  = Mesh.p2;
        bound_Dirichlet2    = Mesh.bound.Diri2;
        max_degree          = 2;
    else
        t2                  = t;
        p2                  = p;
        max_degree          = 1;
    end
    
    bound_Dirichlet     = Mesh.bound.Diri;
    bound_Neumann       = Mesh.bound.Neum;
    

    % Funktionen der Bilinearformen und Linearformen
    func_a      = function_spec.a;
    func_f      = function_spec.f;
    func_g      = function_spec.g;
    deg_Ans     = function_spec.deg_Ans;
    deg_Tes     = function_spec.deg_Tes;
    
    % Es muss sichergestellt werden, dass auf dem Gitter die Knotenpunkte
    % fr entsprechend hohe Grade der Ansatzfunktionen vorliegen.
    assert(max(deg_Ans,deg_Tes)<=max_degree,'Fr Basisfunktionen hherer Ordnung fehlen weitere Knotenpunkte --> t2');
    % Eigentlich werden diese Knotenpunkte nicht gebraucht; weder auf dem
    % Gitter, noch zur Berechnung von Sttzstellen, noch fr Plots, noch
    % fr die Konstruktion der Matrizen.
    %
    % Ich habe sie dennoch reingenommen, um ber die Zeile von p, in
    % welcher der Knoten steht, eine Nummer fr die zugehrige Spalte /
    % Zeile der Systemmatrix zu erhalten.
    
    
    % Aus der Steifigkeitsmatrix und der rechten Seite werden Zeilen und
    % Spalten gelscht, nmlich diejenigen der festgelegten homogenen
    % Dirichlet-Randwert-Freiheitsgrade.
    % Die Anstze sind die Spalten, die Testfunktionen die Zeilen.
    %
    % Wie viele Zeilen und Spalten es gibt, hngt von der Dimensionszahl
    % der jeweiligen Rume ab.
    switch deg_Ans
        case 1
            bound_Dirichlet_Ans     = bound_Dirichlet;
            t_Ans                   = t;
        case 2
            bound_Dirichlet_Ans     = bound_Dirichlet2;
            t_Ans                   = t2;
    end
    
    switch deg_Tes
        case 1
            bound_Dirichlet_Tes     = bound_Dirichlet;
            t_Tes                   = t;
        case 2
            bound_Dirichlet_Tes     = bound_Dirichlet2;
            t_Tes                   = t2;
    end

    
    
    %% Problemdimension
    dim         = size(p,2);
    disp(dim)

    

    %% Quadratur-Methoden initialisieren
    [XI_int,W_int,N_int,XI_ref]                         = Quadratur_Init(dim);
    
    [XI_int_bound,W_int_bound,N_int_bound,XI_ref_bound] = Quadratur_Init(dim-1);
    % Dimensionen der Sttzstellen auffllen
    XI_int_bound = [XI_int_bound,zeros(N_int_bound,1)];
    

    %% Ansatz-Funktionen auf den Integrationsknotenpunkten ausrechnen
    
    % Funktionswerte auf den Sttzstellen der Volumenintegrale
    
    % n_Nodes_Ans     % Anzahl der Freiheitsgrade pro Finitem Element
    switch deg_Ans
        case 1
            switch dim
                case 2
                    n_Nodes_Ans     = 3;     	
                case 3
                    n_Nodes_Ans     = 4;
            end
        case 2
            switch dim
                case 2
                    n_Nodes_Ans     = 6;
                case 3
                    n_Nodes_Ans     = 10;
            end
    end
    
    % Ansatzfunktionen selbst
    Phi_Ans_int    	= zeros(n_Nodes_Ans,N_int);
    for k=1:n_Nodes_Ans
        for i=1:N_int
            Phi_Ans_int(k,i)        = func_ref_Phi(deg_Ans,k,XI_int(i,:));          % Innere Integral-Sttzstellen
        end
    end
    
    Phi_Ans_int_bound   = zeros(n_Nodes_Ans,N_int_bound);
    for k=1:n_Nodes_Ans
        for i=1:N_int_bound
            Phi_Ans_int_bound(k,i) 	= func_ref_Phi(deg_Ans,k,XI_int_bound(i,:));    % Randintegral-Sttzstellen
        end
    end
    

    % Gradienten der Ansatzfunktionen
    hGradPhi_Ans_int    = zeros(dim,n_Nodes_Ans,N_int);

    for k=1:n_Nodes_Ans
        for i=1:N_int            
            hGradPhi_Ans_int(:,k,i)     = func_ref_GradPhi(deg_Ans,k,XI_int(i,:));
        end
    end
    
    hGradPhi_Ans_int_bound  = zeros(dim,n_Nodes_Ans,N_int);

    for k=1:n_Nodes_Ans
        for i=1:N_int_bound
            hGradPhi_Ans_int_bound(:,k,i) 	= func_ref_GradPhi(deg_Ans,k,XI_int_bound(i,:));
        end
    end
    
    
    %% Test-Funktionen auf den Integrationsknotenpunkten ausrechnen
    
    % Funktionswerte auf den Sttzstellen der Volumenintegrale
    % n_Nodes_Ans     % Anzahl der Freiheitsgrade pro Finitem Element
    switch deg_Tes
        case 1
            switch dim
                case 2
                    n_Nodes_Tes     = 3;     	
                case 3
                    n_Nodes_Tes     = 4;
            end
        case 2
            switch dim
                case 2
                    n_Nodes_Tes     = 6;
                case 3
                    n_Nodes_Tes     = 10;
            end
    end
    
    % Ansatzfunktionen selbst
    Phi_Tes_int             = zeros(n_Nodes_Tes,N_int);
    for k=1:n_Nodes_Tes
        for i=1:N_int
            Phi_Tes_int(k,i)        = func_ref_Phi(deg_Tes,k,XI_int(i,:));
        end
    end
    
    Phi_Tes_int_bound   = zeros(n_Nodes_Tes,N_int_bound);
    for k=1:n_Nodes_Tes
        for i=1:N_int_bound
            Phi_Tes_int_bound(k,i) 	= func_ref_Phi(deg_Tes,k,XI_int_bound(i,:));
        end
    end
    

    % Gradienten der Ansatzfunktionen
    hGradPhi_Tes_int    = zeros(dim,n_Nodes_Tes,N_int);

    for k=1:n_Nodes_Tes
        for i=1:N_int            
            hGradPhi_Tes_int(:,k,i) = func_ref_GradPhi(deg_Tes,k,XI_int(i,:));
        end
    end
    
    hGradPhi_Tes_int_bound  = zeros(dim,n_Nodes_Tes,N_int);

    for k=1:n_Nodes_Tes
        for i=1:N_int_bound
            hGradPhi_Tes_int_bound(:,k,i)   = func_ref_GradPhi(deg_Tes,k,XI_int_bound(i,:));
        end
    end
    
    
    %% Finite Elemente durchlaufen
    m = size(t,1);
    if(max_degree == 2)
        n = size(p2,1);
    else
        n = size(p,1);
    end
    
    A = sparse(n,n);
    b = zeros(n,1);
    
  
    % Objekte jedes Schleifendurchlaufs;
    
    a_T = zeros(n_Nodes_Tes,n_Nodes_Ans);    	% in dieser Funktion steht fr jedes finite Element der Wert a( Phi_k1 , Phi_k2 )_{T_j} .
                                                % Die Anzahl der nodalen Basisfunktionen ist n_Nodes .
    
    %GradPhi_int = zeros(dim,n_Nodes,N_int);    % In diesem Array stehen alle Gradientenwerte aller Ansatzfunktionen (n_Nodes) auf allen Quadratur-Sttzstellen .
    %Phi_int     = zeros(n_Nodes,N_int);        % In diesem Array stehen alle Gradientenwerte aller Ansatzfunktionen (n_Nodes) auf allen Quadratur-Sttzstellen .
                                                % (Phi_int wurde bereits oben definiert und mit Werten gefllt)
                                                
    GradPhi_Ans_int             = zeros(dim,n_Nodes_Ans,N_int);
    GradPhi_Tes_int             = zeros(dim,n_Nodes_Tes,N_int);
                                                
    %GradPhi_int_bound = zeros(dim,n_Nodes,N_int_bound);   % Ebenfalls die Gradienten, aber auf den Randwerten .
%     GradPhi_Ans_int_bound       = zeros(dim,n_Nodes_Ans,N_int_bound);
    GradPhi_Tes_int_bound       = zeros(dim,n_Nodes_Tes,N_int_bound);
    
    for j=1:m
        % Beobachtung des Fortschritts.
        disp([j,m])
        
        % Fr den Neumann-Rand wird zwecks schnellerer Abfrage ein
        % Hilfs-Bool 'Neum' erstellt.
        % Der Einfachheit halber bercksichtigen wir nur dann Neumann-Rnder,
        % falls sie genau auf genau einem glatten Randstck vorliegen.
        Neum                = bound_Neumann(t(j,1:dim+1),1);
        if( ones(1,dim+1) * Neum == dim )
            Neum            = true;
        else
            Neum            = false;
        end
       
        % FE-Eckpunkte bestimmen
        X                   = p(t(j,:),:);
        Trafo_Mat           = X(2:dim+1,:) - ones(dim,1) * X(1,:);
        det_Trafo_Mat       = det( Trafo_Mat );
        [Q_TM,R_TM]         = qr( Trafo_Mat );
        h                   = norm(Trafo_Mat,2);
        
        if(Neum)
            % Da die Neumann-Rnder bereits bei der Zuweisung der Randbools
            % umsortiert wurden, liegt der Neumann-Rand falls berhaupt IMMER auf den
            % ersten Eckpunkten des Tetraheders und wird somit immer auf
            % dieselbe Seite des Referenz-Tetraheders transformiert. Auf
            % diese Weise sind keine Fallunterscheidungen ntig.
            Trafo_Mat_bound                 = X(2:dim,:) - ones(dim-1,1) * X(1,:);
            [~,Trafo_Mat_bound]             = qr(Trafo_Mat_bound',0);
            abs_det_Trafo_Mat_bound         = abs( det( Trafo_Mat_bound ) );
        end
        
        % Quadratur-Punkte bestimmen
        X_int               = ones(N_int,1) * X(1,:) + XI_int * Trafo_Mat;
        
        if(Neum)
            X_int_bound   	= ones(N_int_bound,1) * X(1,:) + XI_int_bound * Trafo_Mat;
            h_bound         = norm(Trafo_Mat_bound,2);
        end
        
        
        
        % Gradientenwerte bestimmen
        % ~ ~ ~ Fr die Ansatzfunktionen
        for k = 1:n_Nodes_Ans               % der Funktionsindex
            for i_int = 1:N_int             % die Quadratur-Sttzstelle
                GradPhi_Ans_int(:,k,i_int)  = R_TM \( Q_TM' * hGradPhi_Ans_int(:,k,i_int) ); % (:,k,i_Nodes)
            end
        end
        
%         % normalerweise braucht man keine Randintegrale der Ansatzfunktionen. Daher hier Auskommentierung.
%         if(Neum)
%             GradPhi_Ans_int_bound = 0 * GradPhi_Ans_int_bound;
%             for k = 1:n_Nodes_Ans          	% der Funktionsindex
%                 for i_int = 1:N_int_bound   % die Quadratur-Sttzstelle
%                     GradPhi_Ans_int_bound(:,k,i_int)  = R_TM \( Q_TM' * hGradPhi_Ans_int_bound(:,k,i_int) ); % (dim,k,i_int)
%                 end
%             end
%         end
        % GradPhi_int = [GP_{d,k,i}]
        % mit
        %   d   : Dimension (Gradient ist ein Vektor)
        %   k   : Ansatzfunktionsindex
        %   i   : Integrationsknotenpunkt
        
        % ~ ~ ~ Fr die Testfunktionen
        for k = 1:n_Nodes_Tes              	% der Funktionsindex
            for i_int = 1:N_int             % die Quadratur-Sttzstelle
                GradPhi_Tes_int(:,k,i_int)  = R_TM \( Q_TM' * hGradPhi_Tes_int(:,k,i_int) ); % (:,k,i_Nodes)
            end
        end
        
        if(Neum)
            GradPhi_Tes_int_bound = 0 * GradPhi_Tes_int_bound;
            for k = 1:n_Nodes_Tes         	% der Funktionsindex
                for i_int = 1:N_int_bound   % die Quadratur-Sttzstelle
                    GradPhi_Tes_int_bound(:,k,i_int)  = R_TM \( Q_TM' * hGradPhi_Tes_int_bound(:,k,i_int) ); % (dim,k,i_int)
                end
            end
        end
        % GradPhi_int = [GP_{d,k,i}]
        % mit
        %   d   : Dimension (Gradient ist ein Vektor)
        %   k   : Ansatzfunktionsindex
        %   i   : Integrationsknotenpunkt
        
        
        
        % Funktionswerte von a(.,.) und f(.) auf einen Schlag ausrechnen
        values_a    = func_a_bilin( Phi_Ans_int , GradPhi_Ans_int , Phi_Tes_int , GradPhi_Tes_int , X_int , h ); % = [a_{i,k1,k2}];
        values_f    = func_f_lin(   Phi_Tes_int , GradPhi_Tes_int , X_int , h ); % = [f_{i,k1}];
        
        % im Neumann-Fall Funktionswerte von g(.) auf einen Schlag ausrechen
        if( Neum )
              values_g = func_g_lin( Phi_Tes_int_bound , GradPhi_Tes_int_bound , X_int_bound , h_bound); % = [g_{i_bound,k1}];
        end
        % mit 
        %   i   : Index des Integrationsknotenpunktes
        %   k1  : Index der Testfunktion
        %   k2  : Index der Ansatzfunktion
        
        
        % Linearform-Integrale ausrechnen
        f_T = abs(det_Trafo_Mat) * (W_int' * values_f); % = [f_{k1}]; sind die fertigen Intgralwerte
        % mit
        %   k1  : Index der Testfunktion
        
        % Im Neumann-Fall Randintegral ausrechnen
        if(Neum)
            g_T             = abs_det_Trafo_Mat_bound * (W_int_bound' * values_g);
            f_T             = f_T + g_T;
        end
        
        b(t_Tes(j,:),1) = b(t_Tes(j,:),1) + f_T';
        
        % Biliniearform-Integrale ausrechnen
        a_T = 0*a_T;
        for k1=1:n_Nodes_Tes
            for k2=1:n_Nodes_Ans
                for i_int=1:N_int
                    a_T(k1,k2) = a_T(k1,k2) + W_int(i_int) * values_a(i_int,k1,k2); % = [a_{k1,k2}]; sind die fertigen Integralwerte
                end
            end
        end
        a_T = abs(det_Trafo_Mat) * a_T;
        % mit
        %   k1, k2 wie oben
        A(t_Tes(j,:),t_Ans(j,:)) = A(t_Tes(j,:),t_Ans(j,:)) + a_T;
        
    end
    
    
    A       = A(~bound_Dirichlet_Tes,~bound_Dirichlet_Ans);
    b       = b(~bound_Dirichlet_Tes,1);

end


% Funktion der Bilinearform: Durch getrennte Ansatz- und Testfunktionen
% knnen auch Sattelpunktprobleme unkompliziert implementiert werden.
function [a_T] = func_a_bilin( phiAns , DphiAns , phiTes , DphiTes, X , h )

    global func_a

    [n_Nodes_Tes,N_int]     = size(phiTes);
    [n_Nodes_Ans,~]         = size(phiAns);

    a_T = zeros(N_int,n_Nodes_Tes,n_Nodes_Ans);
    
    for i = 1:N_int
        for k1=1:n_Nodes_Tes
            for k2=1:n_Nodes_Ans
                % Phi_Tes(i,k1)         :   Testfunktion
                % GradPhi_Tes(:,k1,i)   :   Gradient der Testfunktion
                % Phi_Ans(k2,i)         :   Ansatzfunktion
                % GradPhi_Ans(:,k2,i)   :   Gradient der Ansatzfunktion
                % X(i,:)                :   Knotenpunkt-Koordinate
                
                %a_T(i,k1,k2)    = DphiTes(:,k1,i)'*DphiAns(:,k2,i);
                %a_T(i,k1,k2)    = 0.01 * DphiTes(:,k1,i)'*DphiAns(:,k2,i) + (phiTes(k1,i)+h/2*[0.6,0.8]*DphiTes(1:2,k1,i)) * 2*[0.6,0.8]*DphiAns(1:2,k2,i);
                a_T(i,k1,k2) = func_a(phiTes(k1,i),DphiTes(:,k1,i),phiAns(k2,i),DphiAns(:,k2,i),X(i,:),h);  
            end
        end
    end
    
end


% Funktion der rechten Seite, Innenintegral; zwecks Konstruktion einer inhomogenen DGL.
function [f_T] = func_f_lin( phiTes , DphiTes, X , h )
    global func_f
    
    [n_Nodes,N_int] = size(phiTes);
    
    f_T = zeros(N_int,n_Nodes);
    
    for i = 1:N_int
        for k1=1:n_Nodes
            % Phi(i,k1)         :   Testfunktion
            % GradPhi(:,k1,i)   :   Gradient der Testfunktion
            % X(i,:)            :   Knotenpunkt

            %f_T(i,k1)   = phiTes(k1,i);
            f_T(i,k1) = func_f( phiTes(k1,i) , DphiTes(1,k1,i) , X(i,1) , h ); 
        end
    end
    %f_T   = phiTes';
end


% Funktion der rechten Seite, Randintegral; zwecks Konstruktion inh. Neumann-Rnder.
function [g_T] = func_g_lin( phiTes ,DphiTes , X , h )

    global func_g

    [n_Nodes,N_int] = size(phiTes);

    g_T = zeros(N_int,n_Nodes);
    
    for i = 1:N_int
        for k1=1:n_Nodes
            % phiTes(i,k1)     	:   Testfunktion
            % DphiTes(:,k1,i)   :   Gradient der Testfunktion
            % X(i,:)            :   Knotenpunkt
            
            %g_T(i,k1)   = 0; 
            g_T(i,k1) = func_g(phiTes(k1,i),DphiTes(:,k1,i),X(i,:),h); 
        end
    end
    
end


function [res] = func_ref_Phi(deg,k,x)
    dim = size(x,2);
    
    switch dim
        
        case 2
            switch deg
                case 1
                    B = [   
                            1 -1 -1         0 0 0 ;
                            0  1  0         0 0 0 ;
                            0  0  1         0 0 0 
                        ];
                case 2
                    load('Ansatzfunctions/Basis_P2_D2.mat');
            end
            v   = [1;x(1);x(2);x(1)^2;x(1)*x(2);x(2)^2];
            res = B * v;
            res = res(k);
            
            
        case 3
            switch deg
                case 1
                    B = [   
                            1 -1 -1  0      0 0 0 0 0 0;
                            0  1  0  0      0 0 0 0 0 0;
                            0  0  1  0      0 0 0 0 0 0;
                            0  0  0  1      0 0 0 0 0 0
                        ];
                case 2
                    load('Ansatzfunctions/Basis_P2_D3.mat');
            end
            v   = [1;x(1);x(2);x(3);x(1)^2;x(1)*x(2);x(1)*x(3);x(2)^2;x(2)*x(3);x(3)^2];
            res = B * v;
            res = res(k);
            
    end

end


function [res] = func_ref_GradPhi(deg,k,x)
    dim = size(x,2);
    
    % Phi(x)        = B * Monome(x)
    % D_Phi(x)      = B * D_Monome(x)
    % Grad_Phi(x)   = D_Phi(x)'
    
    switch dim
        
        case 2
            switch deg
                case 1
                    B = [   
                            1 -1 -1         0 0 0 ;
                            0  1  0         0 0 0 ;
                            0  0  1         0 0 0 
                        ];
                case 2
                    load('Ansatzfunctions/Basis_P2_D2.mat');
            end
            Dv  =   [
                        0       0;
                        1       0;
                        0       1;
                        2*x(1)  0;
                        x(2)    x(1);
                        0       2*x(2)
                    ];
                
            res = (B * Dv)';
            res = res(:,k);
            
            
        case 3
            switch deg
                case 1
                    B = [   
                            1 -1 -1 -1      0 0 0 0 0 0 ;
                            0  1  0  0      0 0 0 0 0 0 ;
                            0  0  1  0      0 0 0 0 0 0 ;
                            0  0  0  1      0 0 0 0 0 0 
                        ];
                case 2
                    load('Ansatzfunctions/Basis_P2_D3.mat');
            end
            Dv  =   [
                        0       0       0;
                        1       0       0;
                        0       1       0;
                        0       0       1;
                        2*x(1)  0       0;
                        x(2)    x(1)    0;
                        x(3)    0       x(1);
                        0       2*x(2)  0;
                        0       x(3)    x(2);
                        0       0       2*x(3)
                    ];
                
            res = (B * Dv)';
            res = res(:,k);
            
    end

end


%% %%%%%%%%% Quadratur-Formeln: Hier werden einfach nur Tabellen ausgelesen
function [XI_int,W_int,N_int,XI_ref] = Quadratur_Init(dim)

    switch dim
        
        case 1
            % 1D, Referenzlinie [0,1]. Gauss-Legende-4, Order 7, Degree 7
            % Ref: Wiki Deutsch, Gauss-Quadratur
            XI_1d_ref   = [0; 1];
            XI_1d_int   = 0.5 + 0.5 * ...
                          [ 
                            -sqrt(3/7+2/7*sqrt(6/5));
                            -sqrt(3/7-2/7*sqrt(6/5));
                             sqrt(3/7-2/7*sqrt(6/5));
                             sqrt(3/7+2/7*sqrt(6/5))
                          ];
            W_1d_int    = 0.5 * ...
                          [ 
                            (18-sqrt(30))/36;
                            (18+sqrt(30))/36;
                            (18+sqrt(30))/36;
                            (18-sqrt(30))/36
                          ];
            N_1d_int    = 4;
              
        case 2
            % 2D, Referenzdreieck [(0,0),(1,0),(0,1)]. Strang-5, Order 6, Degree 4
            % Quadrature Rules, Per-Olof Persson
            XI_2d_ref   = [0 0; 1 0; 0 1];
            XI_2d_int   = [ 0.816847572980459   0.091576213509771;
                            0.091576213509771   0.816847572980459;
                            0.091576213509771   0.091576213509771;
                            0.108103018168070   0.445948490915965;
                            0.445948490915965   0.108103018168070;
                            0.445948490915965   0.445948490915965   ];
            W_2d_int    = 0.5 * ...
                          [ 0.109951743655322;
                            0.109951743655322;
                            0.109951743655322;
                            0.223381589678011;
                            0.223381589678011;
                            0.223381589678011   ];
            N_2d_int    = 6;
                   
        case 3
            % Quadrature data for tetrahedron
            % Refs
            %  P Keast, Moderate degree tetrahedral quadrature formulas, CMAME 55: 339-348 (1986)
            %  O. C. Zienkiewicz, The Finite Element Method,  Sixth Edition,

            % 3D, Referenztetraeder [(0,0,0),(1,0,0),(0,1,0),(0,0,1)]. ???, Ordnung mindestens 4
            xa=[0.2500000000000000, 0.0000000000000000, 0.3333333333333333, 0.3333333333333333, 0.3333333333333333, ...
                 0.7272727272727273, 0.0909090909090909, 0.0909090909090909, 0.0909090909090909, 0.4334498464263357, ...
                 0.0665501535736643, 0.0665501535736643, 0.0665501535736643, 0.4334498464263357, 0.4334498464263357];
            ya=[0.2500000000000000, 0.3333333333333333, 0.3333333333333333, 0.3333333333333333, 0.0000000000000000, ...
                 0.0909090909090909, 0.0909090909090909, 0.0909090909090909, 0.7272727272727273, 0.0665501535736643, ...
                 0.4334498464263357, 0.0665501535736643, 0.4334498464263357, 0.0665501535736643, 0.4334498464263357];
            za=[0.2500000000000000, 0.3333333333333333, 0.3333333333333333, 0.0000000000000000, 0.3333333333333333, ...
                 0.0909090909090909, 0.0909090909090909, 0.7272727272727273, 0.0909090909090909, 0.0665501535736643, ...
                 0.0665501535736643, 0.4334498464263357, 0.4334498464263357, 0.4334498464263357, 0.0665501535736643];
            wt=[0.1817020685825351, 0.0361607142857143, 0.0361607142857143, 0.0361607142857143, 0.0361607142857143, ...
                 0.0698714945161738, 0.0698714945161738, 0.0698714945161738, 0.0698714945161738, 0.0656948493683187, ...
                 0.0656948493683187, 0.0656948493683187, 0.0656948493683187, 0.0656948493683187, 0.0656948493683187]./6;
            XI_3d_ref   = [0 0 0; 1 0 0; 0 1 0; 0 0 1];         % Knotenpunkte des Referenzelements
            XI_3d_int   = [xa',ya',za'];
            W_3d_int    = wt';
            N_3d_int    = 15;
            clear xa ya za wt;

    end % switch
    

    switch dim
        
        case 1
            XI_int      = XI_1d_int;
            W_int       = W_1d_int;
            N_int       = N_1d_int;
            XI_ref      = XI_1d_ref;
        
        case 2
            XI_int      = XI_2d_int;
            W_int       = W_2d_int;
            N_int       = N_2d_int;
            XI_ref      = XI_2d_ref;
            
        case 3
            XI_int      = XI_3d_int;
            W_int       = W_3d_int;
            N_int       = N_3d_int;
            XI_ref      = XI_3d_ref;
            
    end

end