function [bound_funcs] = Make_bound_funcs()


    %% Description
    %
    % bound_funcs is a struct containing mehsing parameters and domain and
    % boundary information, defined by levelsets. Read Per-Olof Persson 
    % and Gilbert Strang, "A Simple Mesh Generator in MATLAB," SIAM Review
    % Vol. 46 (2) 2004 for details.
    

    %% Beschreibung
    %
    % Nachher:
    %     Die Geometrie des Gebiets wurde durch die Funktion fd beschrieben.
    %   Sie ist negativ im Gebietsinneren und positiv im Gebietsueren.
    %   Auf dem Gebietsrand hat sie den Wert 0.
    %     Die Dirichlet-Randwerte wurden durch die Funktion f_D
    %   beschrieben. Die Bereiche des Gebietsrandes, auf denen f_D <= 0
    %   gilt, sind Dirichlet-Randwerte.
    %
    %     Zum Zwecke der Generierung der Triangulation wird ein
    %   karthesisch-rechteckiger Bereich xbox = [xmin,ymin,zmin;xmax,ymax,zmax]
    %   bentigt, der das Gebiet enthlt.
    %     Fr eckige Gebiete ist es empfehlenswert, darber hinaus feste
    %   Punkte pfix = [px,py,pz;...] auf dem Gebiet festzulgen. Diese
    %   Punkte treten nachher definitiv als Eckpunkte von Tetrahedern auf.
    %
    %
    % Ausgaben
    %   bound_funcs.fd      % Gebietsrand-Funktion
    %   bound_funcs.f_D     % Dirichlet-Gebietsrand-Funktion
    %   bound_funcs.xbox
    %
    % %
    

    
    %% Funktionen erstellen
     
    % Optionale Werte
    pfix    = [];       % Fr den Fall, dass pfix nicht gesetzt wird.
    dim     = 2;        % hier kann man fr dimensionsvariable Testflle die Dimension festlegen.
    
    
    % % Vorgefertigte Funktionen
    BOUNDS  = 'Square_x_m1_Neum';
    %BOUNDS  = 'Square_x_Diri';
    BOUNDS  = 'Circ_half_Neum';
    
    
    % Information:
    % fd    :   function of domain:                 fd(x) < 0   --> x is in domain
    % f_D   :   function of dirichlet boundaries:   f_D(x) < 0  --> x is on Dirichlet bound.
    % dim   :   dimension of domain, only 2 or 3 accepted.
    % pfix, xbox,h    : see Persson et. al.
    switch BOUNDS
        
        case 'Square_x_m1_Neum'
            % [-1,1]^dim
            % bei x_{dim} = 1 ist ein Neumann-Rand
            
            fd      = inline('-1 + transpose(max(transpose(abs(p))))','p');
        
            f_D     = inline('p(:,size(p,2)) - 0.99','p');
            
            xbox    = [-1,-1,-1;1,1,1];
            
            pfix    = [-1,-1,-1;1,-1,-1;-1,1,-1;-1,-1,1;1,1,-1;1,-1,1;-1,1,1;1,1,1]; % alle acht Eckpunkte
            
            if(dim == 2)
                xbox    = xbox(:,1:2);
                pfix    = pfix(:,1:2);
            end
            
            h = 0.2;
        %end case
            
        case 'Square_x_Diri'
            % [-1,1]^dim
            % bei x1 = -1 ist ein Neumann-Rand
            
            fd      = inline('-1 + transpose(max(transpose(abs(p))))','p');
        
            f_D     = inline('-1','p');
            
            xbox    = [-1,-1,-1;1,1,1];
            
            pfix    = [-1,-1,-1;1,-1,-1;-1,1,-1;-1,-1,1;1,1,-1;1,-1,1;-1,1,1;1,1,1]; % alle acht Eckpunkte
            
            if(dim == 2)
                xbox    = xbox(:,1:2);
                pfix    = pfix(:,1:2);
            end   
            
            h = 0.3;
        %end case
        
        case 'Circ_half_Neum'
            % [-1,1]^dim
            % bei x1 = -1 ist ein Neumann-Rand
            
            fd      = inline('-1 + ( p.^2*ones(size(p,2),1) ).^0.5','p');
        
            f_D     = inline('p(:,1)','p');
            
            xbox    = [-1,-1,-1;1,1,1];
            
            pfix    = [0 0 0];

            if(dim == 2)
                xbox    = xbox(:,1:2);
                pfix    = pfix(:,1:2);
            end   
            
            h = 0.2;
        %end case
        
        otherwise
            
            fd      = inline('max(-1+(p(:,1).^2).^0.5,-3+((p(:,2)-1).^2).^0.5)','p');
            f_D     = inline('1+p(:,2)','p');
            
            xbox    = [-2,-2;2,4];
            
            dim     = 2;
            
            h = 0.4;
            
        %end case
        
    end

    
    
    %% Ausgaben einpacken
    bound_funcs.fd      = fd;
    bound_funcs.f_D     = f_D;
    bound_funcs.xbox    = xbox;
    bound_funcs.pfix    = pfix;
    bound_funcs.h       = h;


end