function [Mesh] = Mesh_Enricher(Mesh,deg)

    %% Description
    % This routine extends the number of nodes of the mesh.
    %
    % The FE code allows for finite element functions of polynomials of
    % degree 1 and 2. Only for degree 2 you need this routine to define
    % extra nodal degrees of freedom.
    %
    % NOTICE: This routine is inefficient due to reallocation.


    %% Beschreibung
    %
    % Eingaben:
    %   Mesh.t      % Dreiecksliste
    %   Mesh.p      % Liste der Dreiecks-Eckpunkte
    %
    %   deg         % Ansatz-/Testfunktionen-Grad.  Bisher in {1,2}
    %
    %
    % Vorher
    %   Eine Triangulierung liegt vor.
    %
    % Nachher:
    %   Es wurde eine Liste p2 globaler Knotenpunkte im Gebiet der 
    %   Triangulierung ergnzt. Es wurde eines Dreiecksliste t2
    %   geschrieben, in der fr jedes Dreieck alle seine Punkte aus p2
    %   aufgefhrt sind.
    %
    %   Die Punkte aus p2 geben die Knoten fr eine nodale Basis von
    %   Ansatz-/Testfunktionen der Ordnung deg wieder.
    %
    %
    % Ausgaben
    %   Mesh.t      % Dreiecksliste
    %   Mesh.p      % Liste der Dreiecks-Eckpunkte
    %
    %   Mesh.t2     % Dreiecksliste der erweiterten Punkteliste
    %   Mesh.2      % erweiterte Punkteliste
    
    
    %% Eingaben auspacken
    p   = Mesh.p;
    t   = Mesh.t;
    
    
    switch deg
        
        case 1
            % keine Erweiterung
            p2     = p;
            t2     = t;
        % end case 
            
        case 2
            % erweiterte Knotenlisten erzeugen (ber Mittelwerte der Eckpunkte)
            dim = size(p,2);
            switch dim

                case 2
                    [t2,p2] = Triangulation_Enricher_2D(t,p);

                case 3
                    [t2,p2] = Triangulation_Enricher_3D(t,p);

            end % switch dim
        % end case
        
    end % switch deg

    %% Ausgaben einpacken
    Mesh.p2     = p2;
    Mesh.t2     = t2;

end


function [t2,p2] = Triangulation_Enricher_2D(t,p)

    M   = size(t,1);
    N   = size(p,1);
    
    for j=1:M
        disp([j,M]);
        
        P       = p(t(j,1:3),:);    % = [x1;x2;...x_{dim+1}];
        
        p4      = 0.5 * (P(1,:)+P(2,:));
        [flag,number] = isPointInP(p4,p);
        if(flag)
            t(j,4)  = number;
        else
            p       = [p;p4];
            N       = N + 1;
            t(j,4)  = N;
        end
        
        p5      = 0.5 * (P(1,:)+P(3,:));
        [flag,number] = isPointInP(p5,p);
        if(flag)
            t(j,5)  = number;
        else
            p       = [p;p5];
            N       = N + 1;
            t(j,5)  = N;
        end 

        p6      = 0.5 * (P(2,:)+P(3,:));
        [flag,number] = isPointInP(p6,p);
        if(flag)
            t(j,6)  = number;
        else
            p       = [p;p6];
            N       = N + 1;
            t(j,6)  = N;
        end
        
    end
    
    t2 = t;
    p2 = p;

end


function [t2,p2] = Triangulation_Enricher_3D(t,p)

    M   = size(t,1);
    N   = size(p,1);
    
    for j=1:M
        disp([j,M]);
        
        P       = p(t(j,1:4),:);    % = [x1;x2;...x_{dim+1}];
        
        p5      = 0.5 * (P(1,:)+P(2,:));
        [flag,number] = isPointInP(p5,p);
        if(flag)
            t(j,5)  = number;
        else
            p       = [p;p5];
            N       = N + 1;
            t(j,5)  = N;
        end
        
        p6      = 0.5 * (P(1,:)+P(3,:));
        [flag,number] = isPointInP(p6,p);
        if(flag)
            t(j,6)  = number;
        else
            p       = [p;p5];
            N       = N + 1;
            t(j,6)  = N;
        end 
        
        p7      = 0.5 * (P(1,:)+P(4,:));
        [flag,number] = isPointInP(p7,p);
        if(flag)
            t(j,7)  = number;
        else
            p       = [p;p7];
            N       = N + 1;
            t(j,7)  = N;
        end
        
        p8      = 0.5 * (P(2,:)+P(3,:));
        [flag,number] = isPointInP(p8,p);
        if(flag)
            t(j,8)  = number;
        else
            p       = [p;p8];
            N       = N + 1;
            t(j,8)  = N;
        end
        
        p9      = 0.5 * (P(2,:)+P(4,:));
        [flag,number] = isPointInP(p9,p);
        if(flag)
            t(j,9)  = number;
        else
            p       = [p;p9];
            N       = N + 1;
            t(j,9)  = N;
        end
        
        p10     = 0.5 * (P(3,:)+P(4,:));
        [flag,number] = isPointInP(p10,p);
        if(flag)
            t(j,10) = number;
        else
            p       = [p;p10];
            N       = N + 1;
            t(j,10) = N;
        end
        
    end
    
    t2 = t;
    p2 = p;

end


function [booo,index] = isPointInP(p1,p)

    N       = size(p,1);
    booo    = false;
    index   = 0;
    
    for i=1:N
        if( norm(p1-p(i,:)) < 1e-6 )
            booo    = true;
            index   = i;
            break;
        end
    end

end