function [Mesh] = Solution_Plotter_Scalar(x,Mesh)

    %% Description
    % If vector x contains a scalar field belonging to the mesh points in Mesh.p, then
    % this routine visualizes this field over the finite elements by a
    % surface or color plot.
    

    %% Beschreibung
    %
    % Eingaben:
    %   x                   % Lsung des Gleichungssystems A*x=b der Galerkin-Diskretisierung.
    %   Mesh.p              % Tetraheder-Eckpunkte
    %   Mesh.t              % Tetraheder-Liste
    %   Mesh.bound.Diri     % Liste ob p(i,:) ein Dirichlet-Randwert ist.
    %
    %
    % Nachher:
    %   Die skalaren Lsungswerte wurden den Knotenpunkten aus p und den
    %   Dreiecken t zugewiesen. Dazu wurden skalare Werte p_sol fr die
    %   Knotenpunkte erstellt und skalare Werte t_sol als Mittelwerte fr
    %   die Dreiecke.
    %
    %   Der Dimensionsfall wird automatisch erkannt und das Ergebnis wird
    %   auf den Zellen geplottet.
    %
    %
    % Ausgaben (nur zustzliche)
    %   Mesh.p_sol          % Lsungswerte auf den Knotenpunkten
    %   Mesh.t_sol          % gemittelter Lsungswert auf jedem Dreieck.
    %    
    % %
    

    %% Eingaben auspacken
    t                   = Mesh.t;
    p                   = Mesh.p;
    bound_Dirichlet     = Mesh.bound.Diri;

    
    %% Graphische Ausgabe der Lsung
    % Da Matlabs In-Built nur stckweise lineare Funktionen ad-hoc plotten
    % kann, reduzieren wir die Lsung auf die Werte auf den Randpunkten.
    % Dadurch kann sie wie gehabt als stckweise linear ausgegeben werden.
    %
    % Die Farbgebung erfolgt als Mittelwert der Funktionswerte auf
    % lediglich den Eckpunkten des jeweiligen Elements.
    %
    % Die Funktionswerte, die den Gittereckpunkten p zuzuweisen sind,
    % werden in p_sol geschrieben.
    dim = size(p,2);
    
    switch dim
        
        case 2
            % Lsung graphisch ausgeben
            N = size(p,1);
            M = size(t,1);

            p_sol = zeros(N,1);

            p_sol(~bound_Dirichlet,1) = x(1:size(p_sol(~bound_Dirichlet),1),1);

            
            t_sol      = zeros(M,1);
            g           = ones(1,3)/3;
            for j=1:M
                t_sol(j,1) = g * p_sol(t(j,:),1);
            end
            
            trisurf(t,p(:,1),p(:,2),p_sol,t_sol);
            view(2); axis equal; axis tight; %axis off;
            colorbar;
            
        
        case 3
            % Lsung graphisch ausgeben
            N = size(p,1);
            M = size(t,1);
            
            p_sol   = zeros(N,1);
            
            p_sol(~bound_Dirichlet,1) = x(1:size(p_sol(~bound_Dirichlet),1),1);
            
            % Farbwerte der Tetraheder festlegen
            t_sol  	= zeros(M,1);
            for j=1:M
                t_sol(j,1) = ones(1,4)/4 * p_sol(t(j,:),1);
            end
            
            % Tetraheder lschen, um das Gebiet "aufzuschneiden"
            Tetra_to_Hold = true(M,1);
            for j=1:M
                Tetra_to_Hold(j,1) = min( p(t(j,:),1) ) < 0;
            end
            
            t       = t(Tetra_to_Hold,:);
            t_sol  	= t_sol(Tetra_to_Hold,1);
            
            tetramesh(t,p,t_sol');
            axis equal; axis tight; %axis off;
            colorbar;
            
    end % switch
    
    
    %% Ausgaben einpacken
    Mesh.p_sol      = p_sol;
    Mesh.t_sol      = t_sol;


end