function [Mesh]=distmeshnd(fdist,fh,h,box,fix,varargin)

    %DISTMESHND N-D Mesh Generator using Distance Functions.
    %   [P,T]=DISTMESHND(FDIST,FH,H,BOX,FIX,FDISTPARAMS)
    %
    %      P:           Node positions (NxNDIM)
    %      T:           Triangle indices (NTx(NDIM+1))
    %      FDIST:       Distance function
    %      FH:          Edge length function
    %      H:           Smallest edge length
    %      BOX:         Bounding box [xmin,xmax;ymin,ymax; ...] (NDIMx2)
    %      FIX:         Fixed node positions (NFIXxNDIM)
    %      FDISTPARAMS: Additional parameters passed to FDIST
    %
    %   Example: Unit ball
    %      dim=3;
    %      d=inline('sqrt(sum(p.^2,2))-1','p');
    %      [p,t]=distmeshnd(d,0,0.2,[-ones(1,dim);ones(1,dim)],[]);
    %
    %   See also: DISTMESH2D, DELAUNAYN, TRIMESH, MESHDEMOND.

    %   Copyright (C) 2004-2012 Per-Olof Persson. See COPYRIGHT.TXT for details.
    %
    %   Diese ursprngliche Version wurde von Martin 2015 zwecks Integration in ein FEM-Programm verndert.
    %
    %

    % /// Beginn nderungen Martin
    if(nargin == 1)
        bound_funcs     = fdist;
        fdist           = bound_funcs.fd;
        if(isfield(bound_funcs,'fh'))
            fh          = bound_funcs.fh;
        else
            fh          = 0;
        end
        if(isfield(bound_funcs,'h'))
            h           = bound_funcs.h;
        else
            h           = 0.1;
        end
        box             = bound_funcs.xbox;
        fix             = bound_funcs.pfix;
    end
    
    fdist
    fh
    h
    box
    fix
    
    if(fh == 0)
        fh  = @huniform;
    end
    % \\\ Ende nderungen Martin

    dim=size(box,2);
    ptol=.001; ttol=.1; L0mult=1+.4/2^(dim-1); deltat=.1; geps=1e-1*h; deps=sqrt(eps)*h;

    % 1. Create initial distribution in bounding box
    if dim==1
      p=(box(1):h:box(2))';
    else
      cbox=cell(1,dim);
      for ii=1:dim
        cbox{ii}=box(1,ii):h:box(2,ii);
      end
      pp=cell(1,dim);
      [pp{:}]=ndgrid(cbox{:});
      p=zeros(prod(size(pp{1})),dim);
      for ii=1:dim
        p(:,ii)=pp{ii}(:);
      end
    end

    % 2. Remove points outside the region, apply the rejection method
    p=p(feval(fdist,p,varargin{:})<geps,:);
    r0=feval(fh,p);
    p=[fix; p(rand(size(p,1),1)<min(r0)^dim./r0.^dim,:)];
    N=size(p,1);

    count=0; counter = 0;
    p0=inf;
    while 1
        counter = counter + 1

      % 3. Retriangulation by Delaunay
      if max(sqrt(sum((p-p0).^2,2)))>ttol*h
        p0=p;
        t=delaunayn(p);
        pmid=zeros(size(t,1),dim);
        for ii=1:dim+1
          pmid=pmid+p(t(:,ii),:)/(dim+1);
        end
        t=t(feval(fdist,pmid,varargin{:})<-geps,:);
        % 4. Describe each edge by a unique pair of nodes
        pair=zeros(0,2);
        localpairs=nchoosek(1:dim+1,2);
        for ii=1:size(localpairs,1)
          pair=[pair;t(:,localpairs(ii,:))];
        end
        pair=unique(sort(pair,2),'rows');
        % 5. Graphical output of the current mesh
        if dim==2
          trimesh(t,p(:,1),p(:,2),zeros(N,1))
          view(2);axis equal; axis off ;drawnow
        elseif dim==3
          if mod(count,5)==0
            simpplot(p,t,'p(:,2)>0');
            title(['Retriangulation #',int2str(count)])
            drawnow
          end
        else
          disp(sprintf('Retriangulation #%d',count))
        end
        count=count+1;
      end

      % 6. Move mesh points based on edge lengths L and forces F
      bars=p(pair(:,1),:)-p(pair(:,2),:);
      L=sqrt(sum(bars.^2,2));
      L0=feval(fh,(p(pair(:,1),:)+p(pair(:,2),:))/2);
      L0=L0*L0mult*(sum(L.^dim)/sum(L0.^dim))^(1/dim);
      F=max(L0-L,0);
      Fbar=[bars,-bars].*repmat(F./L,1,2*dim);
      dp=full(sparse(pair(:,[ones(1,dim),2*ones(1,dim)]), ...
                     ones(size(pair,1),1)*[1:dim,1:dim], ...
                     Fbar,N,dim));
      dp(1:size(fix,1),:)=0;
      p=p+deltat*dp;

      % 7. Bring outside points back to the boundary
      d=feval(fdist,p,varargin{:}); ix=d>0;
      gradd=zeros(sum(ix),dim);
      for ii=1:dim
        a=zeros(1,dim);
        a(ii)=deps;
        d1x=feval(fdist,p(ix,:)+ones(sum(ix),1)*a,varargin{:});
        gradd(:,ii)=(d1x-d(ix))/deps;
      end
      p(ix,:)=p(ix,:)-d(ix)*ones(1,dim).*gradd;

      % 8. Termination criterion
      maxdp=max(deltat*sqrt(sum(dp(d<-geps,:).^2,2)));
      %if maxdp<ptol*h, break; end
      if counter > 500, break; end % // nderungen Martin
    end

    
    % // nderungen Martin
    %% Ausgaben einpacken
    Mesh.t      = t;
    Mesh.p      = p;

end



function h=huniform(p,varargin) % Uniform h(x,y) distribution
    h=ones(size(p,1),1);
end


function simpplot(p,t,expr,bcol,icol,nodes,tris)

    %   Copyright (C) 2004-2012 Per-Olof Persson. See COPYRIGHT.TXT for details.

    dim=size(p,2);
    switch dim
     case 2
      if nargin<4 | isempty(bcol), bcol=[.8,.9,1]; end
      if nargin<5 | isempty(icol), icol=[0,0,0]; end
      if nargin<6, nodes=0; end
      if nargin<7, tris=0; end

      trimesh(t,p(:,1),p(:,2),0*p(:,1),'facecolor',bcol,'edgecolor','k');
      if nodes==1
        line(p(:,1),p(:,2),'linest','none','marker','.','col',icol,'markers',24);
      elseif nodes==2
        for ip=1:size(p,1)
          txtpars={'fontname','times','fontsize',12};
          text(p(ip,1),p(ip,2),num2str(ip),txtpars{:});
        end
      end
      if tris==2
        for it=1:size(t,1)
          pmid=mean(p(t(it,:),:),1);
          txtpars={'fontname','times','fontsize',12,'horizontala','center'};
          text(pmid(1),pmid(2),num2str(it),txtpars{:});
        end
      end
      view(2)
      axis equal
      axis off
      ax=axis;axis(ax*1.001);
     case 3
      if nargin<4 | isempty(bcol), bcol=[.8,.9,1]; end
      if nargin<5 | isempty(icol), icol=[.9,.8,1]; end

      if size(t,2)==4
        tri1=surftri(p,t);
        if nargin>2 & ~isempty(expr)
          incl=find(eval(expr));
          t=t(any(ismember(t,incl),2),:);
          tri1=tri1(any(ismember(tri1,incl),2),:);
          tri2=surftri(p,t);
          tri2=setdiff(tri2,tri1,'rows');
          h=trimesh(tri2,p(:,1),p(:,2),p(:,3));
          set(h,'facecolor',icol,'edgecolor','k');
          hold on
        end
      else
        tri1=t;
        if nargin>2 & ~isempty(expr)
          incl=find(eval(expr));
          tri1=tri1(any(ismember(tri1,incl),2),:);
        end
      end
      h=trimesh(tri1,p(:,1),p(:,2),p(:,3));
      hold off
      set(h,'facecolor',bcol,'edgecolor','k');
      axis equal
      cameramenu
     otherwise
      error('Unimplemented dimension.');
    end
end



function tri=surftri(p,t)
    %SURFTRI Find surface triangles from tetrahedra mesh
    %   TRI=SURFTRI(P,T)

    %   Copyright (C) 2004-2012 Per-Olof Persson. See COPYRIGHT.TXT for details.

    % Form all faces, non-duplicates are surface triangles
    faces=[t(:,[1,2,3]);
           t(:,[1,2,4]);
           t(:,[1,3,4]);
           t(:,[2,3,4])];
    node4=[t(:,4);t(:,3);t(:,2);t(:,1)];
    faces=sort(faces,2);
    [foo,ix,jx]=unique(faces,'rows');
    vec=histc(jx,1:max(jx));
    qx=find(vec==1);
    tri=faces(ix(qx),:);
    node4=node4(ix(qx));

    % Orientation
    v1=p(tri(:,2),:)-p(tri(:,1),:);
    v2=p(tri(:,3),:)-p(tri(:,1),:);
    v3=p(node4,:)-p(tri(:,1),:);
    ix=find(dot(cross(v1,v2,2),v3,2)>0);
    tri(ix,[2,3])=tri(ix,[3,2]);
end