function [Mesh] = getBoundary(bound_funcs,Mesh)

    %% Description
    % Given (extended) mesh receives nodal data of boundary type.
    % function generates a plot of mesh with red mesh nodes for Dirichlet 
    % type and blue nodes for Neumann type boundaries.
    %
    % Copyright (c) 2015-2023 Martin Neuenhofen.

    %% Beschreibung
    %
    % Eingaben:
    %   bound_funcs.fd      % Distanzfunktion fr Gebietsrand, im Inneren negativ.
    %   bound_funcs.f_D     % Funktion, die auf allen Dirichlet-Rndern positiv ist.
    %
    %   Mesh.p              % Koordinaten von Dreiecks-Eckpunkten
    %   Mesh.t              % Dreiecks-Liste, die auf p verweist.
    %
    %  Optional:
    %   Mesh.p2             % weitere Koordinaten von Punkten auf Dreiecken, enthlt p.
    %   Mesh.t2             % Dreiecks-Liste, die auf p2 verweist.
    %
    %
    % Vorher:
    %   Mesh ist bisher eine Struktur, die lediglich Dreiecke t und deren
    %   Eckpunkte p (in N by dim) enthlt.
    %
    %   Mesh kann optional weitere Punkte p2 enthalten, welche die weiteren
    %   nodalen Basisfunktionen festlegen. Dann muss es auch eine Liste t2
    %   haben, in der fr jedes Dreieck alle Punkte aus p2 stehen.
    %
    % Nachher:
    %   In Mesh wurden unter '.bound' Listen '.Diri', '.Neum' und 'all' ergnzt,
    %   welche fr jeden Knoten aus p festlegen, ob dieser ein Dirichlet-,
    %   ein Neumann-Rand oder grundstzlich ein Rand ist.
    %
    %   Falls eine Liste p2 in Mesh enthalten war, so werden '.bound' die 
    %   Listen '.Diri2', '.Neum2' und 'all2' fr p2 ergnzt.
    %
    %
    % Ausgaben:
    %   Mesh.p              % Koordinaten von Dreiecks-Eckpunkten
    %   Mesh.t              % Dreiecks-Liste, die auf p verweist, nach Neumann-Randflchen sortiert.
    %   Mesh.bound.all      % Bool-Liste, ob p(i,:) auf dem Gebietsrand liegt.
    %   Mesh.bound.Diri     % Bool-Liste, ob p(i,:) auf dem Dirichlet-Rand liegt.
    %   Mesh.bound.Neum     % Bool-Liste, ob p(i,:) auf dem Neumann-Rand liegt.
    %
    %  Optional:
    %   Mesh.p2             % weitere Koordinaten von Punkten auf Dreiecken, enthlt p.
    %                       % die Eintrge von p2 mssen nicht umsortiert werden, es geht nur um die Integral-Sttzstellen.
    %   Mesh.t2             % Dreiecks-Liste, die auf p2 verweist.
    %   Mesh.bound.all2     % Bool-Liste, ob p2(i,:) auf dem Gebietsrand liegt.
    %   Mesh.bound.Diri2    % Bool-Liste, ob p2(i,:) auf dem Dirichlet-Rand liegt.
    %   Mesh.bound.Neum2    % Bool-Liste, ob p2(i,:) auf dem Neumann-Rand liegt.
    %
    % %
    

    %% Eingaben auspacken
    % Randfunktionen
    fd                      = bound_funcs.fd;
    f_Dirichlet             = bound_funcs.f_D;
    
    % Gitter
    t       = Mesh.t;
    p       = Mesh.p;
    
    if(isfield(Mesh,'p2'))
       	p2                  = Mesh.p2;

        degree              = 2;
    else
        degree              = 1;
    end
    
    
    
    %% Randknotenpunkte identifizieren
    tol                 = 1e-3;

    boundary            = feval(fd,p).^2 < tol^2;

    bound_Dirichlet     = (((feval(f_Dirichlet,p) <  tol) * 1) .* boundary) == 1;
    
    bound_Neumann       = (boundary .* (1-bound_Dirichlet)) == 1;
    
    if(degree > 1)
        boundary2           = feval(fd,p2).^2 < tol^2;
        bound_Dirichlet2    = (((feval(f_Dirichlet,p2) <  tol) * 1) .* boundary2) == 1;
        bound_Neumann2      = (boundary2 .* (1-bound_Dirichlet2)) == 1;
    else
        bound_Dirichlet2    = 0;
    end
    
    
    %% Neumann-Randflchen einheitlich anordnen
    % Das Netz wird so berschrieben, dass die Neumann-Knoten als erste in
    % t auftreten.
    dim     = size(p,2);
    M       = size(t,1);
    
    for j=1:M
        
        Neum                = bound_Neumann(t(j,1:dim+1),1);
        if( ones(1,dim+1) * Neum == dim )
            k_last          = 1:dim+1;
            k_last          = k_last(~Neum);
            % Dreieckstausch der (dim+1)-ten Ecke mit der k_last-ten
            tmp             = t(j,dim+1);
            t(j,dim+1)      = t(j,k_last);
            t(j,k_last)     = tmp;
        end
    end

    
    %% Plotten
    % Ab hier wird nur noch geplottet
    switch dim
        case 2
            trimesh(t,p(:,1),p(:,2),0*p(:,1));
            view(2); axis equal; axis tight;
            hold on;
            
            % jeden Punkt im Gitter plotten
            if(isfield(Mesh,'p2'))
                plot(p2(~boundary2,1),p2(~boundary2,2),'green.');
                plot(p2(boundary2,1),p2(boundary2,2),'blacko');
                plot(p2(bound_Dirichlet2,1),p2(bound_Dirichlet2,2),'red.');
                plot(p2(bound_Neumann2,1),p2(bound_Neumann2,2),'blue.');
            else
                plot(p(~boundary,1),p(~boundary,2),'green.');
                plot(p(boundary,1),p(boundary,2),'blacko');
                plot(p(bound_Dirichlet,1),p(bound_Dirichlet,2),'red.');
                plot(p(bound_Neumann,1),p(bound_Neumann,2),'blue.');
            end
            title('yellow : inner node,  black : boundary,  red : Dirichlet,  blue : Neumann','fontsize',14);
            
            
            
        case 3
            figure
%             tetramesh(t,p,0*t);
            hold on;
            
            % Randpunkte einfrben
            plot3(p(bound_Dirichlet,1),p(bound_Dirichlet,2),p(bound_Dirichlet,3),'red.');
            plot3(p(bound_Neumann,1),p(bound_Neumann,2),p(bound_Neumann,3),'blue.');
            axis equal; axis tight;
            xlabel('x'); ylabel('y'); zlabel('z');
    end % switch
    
    
    %% Ausgaben einpacken
    Mesh.t                  = t;    % Knotenindizes umsortiert, auerdem ggf. neue Knoten ergnzt .
    %Mesh.p                 = p;    % keine Vernderung
    if(degree > 1)
        Mesh.p2             = p2;
        Mesh.bound.Diri2    = bound_Dirichlet2;     % wird zur Elimination von unfreien Gleichungssystem-Anteilen gebraucht
        Mesh.bound.Neum2    = bound_Neumann2;       % wird eigentlich nicht gebraucht
        Mesh.bound.all2     = boundary2;            % wird eigentlich nicht gebraucht
    end
    Mesh.bound.Diri         = bound_Dirichlet;      % wird nur im Plotter gebraucht
    Mesh.bound.Neum         = bound_Neumann;        % wird zur Identifizierung von Neumann-Tetrahedern gebraucht
    Mesh.bound.all          = boundary;             % wird eigentlich nicht gebraucht
    

end